"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.nextEventsCollectApi = exports.collectApiHandler = exports.collectEvents = exports.eventCollector = exports.parsePublicUrlNodeApi = exports.getCookie = exports.getAllHeaders = exports.getHeader = void 0;
const server_1 = require("next/server");
const tools_1 = require("./tools");
const index_1 = require("./index");
const next_shim_1 = require("./next-shim");
const log_1 = require("./log");
/**
 * A facade function that gets header from NextRequest or NextApiRequest.
 *
 * In case of multiple headers with the same name, values will be separated by `,`
 */
function getHeader(req, h) {
    if (req instanceof server_1.NextRequest) {
        return req.headers.get(h.toLowerCase()) || undefined;
    }
    else {
        const val = req.headers[h.toLowerCase()];
        if (val && Array.isArray(val)) {
            return val.join(",");
        }
        else {
            return val;
        }
    }
}
exports.getHeader = getHeader;
function getAllHeaders(req) {
    return (req instanceof server_1.NextRequest ? [...req.headers.entries()] : [...Object.entries(req.headers)])
        .sort(([name1], [name2]) => name1.localeCompare(name2))
        .reduce((acc, [name, val]) => ({ ...acc, [name]: Array.isArray(val) ? val.join(",") : val }), {});
}
exports.getAllHeaders = getAllHeaders;
/**
 * A facade function that gets cookie val from NextRequest or NextApiRequest
 */
function getCookie(req, name) {
    return req instanceof server_1.NextRequest ? req.cookies.get(name) : req.cookies[name];
}
exports.getCookie = getCookie;
function parsePublicUrlNodeApi(req) {
    throw new Error("Not implemeted");
}
exports.parsePublicUrlNodeApi = parsePublicUrlNodeApi;
function isSystemRequest(req) {
    return req.nextUrl.pathname.indexOf("/_next/") === 0;
}
function eventCollector(opts) {
    const eventTypeMaps = (0, index_1.parseEventTypesMap)(opts.eventTypes);
    return {
        async nextApiHandler(req, res, props) {
            const reqResShim = next_shim_1.nextApiShim;
            try {
                const clientSideProps = req.body;
                const clientSideEventProps = clientSideProps?.event || {};
                const extraProps = (0, index_1.safeCall)(() => getDynamicOption(props?.extend, {})(req, res, clientSideEventProps), "nextApiHandler.props", {});
                const url = reqResShim.parsePublicUrl(req);
                const eventType = clientSideProps ? clientSideProps.event.eventType : eventTypeMaps(url.path);
                if (!eventType) {
                    return res;
                }
                const anonymousId = reqResShim.getAnonymousId(getDynamicOption(props?.cookieName, next_shim_1.defaultCookieName)(req, res, clientSideEventProps), getDynamicOption(props?.cookieDomain, undefined)(req, res, clientSideEventProps), req, res, url);
                const originalPageEvent = reqResShim.getPageEvent(eventType, url, anonymousId, req);
                const pageEvent = (0, tools_1.deepMerge)(originalPageEvent, clientSideEventProps, extraProps);
                const drivers = (0, index_1.parseDriverShortcut)(opts.drivers);
                if ((0, index_1.isDebug)()) {
                    log_1.consoleLog.log(`Sending page event to ${drivers.map(d => d.type)}: ${JSON.stringify(pageEvent)}`);
                }
                const driversResults = {};
                const allDrivers = [];
                for (const driver of drivers) {
                    const startTime = new Date().getTime();
                    if ((0, index_1.isDebug)()) {
                        log_1.consoleLog.log(`Sending data to ${driver.type}(${driver.opts ? JSON.stringify(driver.opts) : ""})`);
                    }
                    const promise = new Promise(resolve => {
                        (0, index_1.getEventHandler)(driver)(pageEvent, { fetch, log: log_1.consoleLog })
                            .catch(e => {
                            driversResults[driver.type] = `error/${new Date().getTime() - startTime}ms`;
                            if (opts.errorHandler) {
                                opts.errorHandler(driver.type, e);
                            }
                            else {
                                log_1.consoleLog.warn(`Can't send data to ${driver.type}: ${e?.message || "unknown error"}`, e);
                            }
                            resolve();
                        })
                            .then(r => {
                            driversResults[driver.type] = `ok/${new Date().getTime() - startTime}ms`;
                            if ((0, index_1.isDebug)()) {
                                log_1.consoleLog.info(`${driver.type}(${driver.opts ? JSON.stringify(driver.opts) : ""} - api) finished successfully ${new Date().getTime() - startTime}ms. Result: ${JSON.stringify(r)}`);
                            }
                            resolve();
                        });
                    });
                    allDrivers.push(promise);
                }
                await Promise.all(allDrivers);
                res.json({ ok: true, ...driversResults });
            }
            catch (e) {
                res.json({ ok: false, error: `${e?.message || "Unknown error"}` });
                log_1.consoleLog.error(e?.message || "Unknown error", e);
            }
        },
        async nextJsPageMiddleware(req, res, props = undefined) {
            if (!props?.processSystemRequests && isSystemRequest(req)) {
                if ((0, index_1.isDebug)()) {
                    log_1.consoleLog.debug(`Skip system request ${req.nextUrl.pathname}`);
                }
                return res;
            }
            const reqResShim = next_shim_1.pageMiddlewareShim;
            try {
                const extraProps = (0, index_1.safeCall)(() => getDynamicOption(props?.extend, {})(req, res, {}), "nextJsPageMiddleware.props", {});
                const url = next_shim_1.pageMiddlewareShim.parsePublicUrl(req);
                const eventType = eventTypeMaps(url.path);
                if (!eventType) {
                    return res;
                }
                const anonymousId = reqResShim.getAnonymousId(getDynamicOption(props?.cookieName, next_shim_1.defaultCookieName)(req, res, {}), getDynamicOption(props?.cookieDomain, undefined)(req, res, {}), req, res, url);
                const originalPageEvent = reqResShim.getPageEvent(eventType, url, anonymousId, req);
                const pageEvent = (0, tools_1.deepMerge)(originalPageEvent, extraProps);
                const drivers = (0, index_1.parseDriverShortcut)(opts.drivers);
                if ((0, index_1.isDebug)()) {
                    log_1.consoleLog.log(`Sending page event to ${drivers.map(d => d.type)}`, pageEvent);
                }
                for (const driver of drivers) {
                    if ((0, index_1.isDebug)()) {
                        log_1.consoleLog.log(`Sending data to ${driver.type}(${driver.opts ? JSON.stringify(driver.opts) : ""})`);
                    }
                    const startTime = new Date().getTime();
                    (0, index_1.getEventHandler)(driver)(pageEvent, { fetch, log: log_1.consoleLog })
                        .catch(e => {
                        if (opts.errorHandler) {
                            opts.errorHandler(driver.type, e);
                        }
                        else {
                            log_1.consoleLog.warn(`[WARN] Can't send data to ${driver.type}: ${e?.message || "uknown error"}`, e);
                        }
                    })
                        .then(r => {
                        if ((0, index_1.isDebug)()) {
                            log_1.consoleLog.debug(`${driver.type}(${driver.opts ? JSON.stringify(driver.opts) : ""} - middleware) finished successfully in ${new Date().getTime() - startTime}ms. Result: ${JSON.stringify(r)}`);
                        }
                    });
                }
                return res;
            }
            catch (e) {
                log_1.consoleLog.error(e?.message || "Unknown error", e);
                return res;
            }
        },
    };
}
exports.eventCollector = eventCollector;
function getDynamicOption(o, defaultVal) {
    if (o === undefined) {
        return () => defaultVal;
    }
    else if (typeof o === "function") {
        return o;
    }
    return () => o;
}
function collectEvents(opts) {
    return async (request, event) => {
        const response = opts.middleware
            ? (await opts.middleware(request, event)) || server_1.NextResponse.next()
            : server_1.NextResponse.next();
        return eventCollector(opts).nextJsPageMiddleware(request, response, opts);
    };
}
exports.collectEvents = collectEvents;
function collectApiHandler(opts) {
    const collector = eventCollector(opts);
    return async (request, response) => {
        await collector.nextApiHandler(request, response, opts);
    };
}
exports.collectApiHandler = collectApiHandler;
/**
 * @deprecated use collectEvents instead
 */
exports.nextEventsCollectApi = collectApiHandler;
